# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python imports
from types import ListType
# zope imports
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from OFS.PropertyManager import PropertyManager
from OFS.ObjectManager import ObjectManager
from OFS.SimpleItem import SimpleItem
from AccessControl.Permissions import view, view_management_screens

from Products.PageTemplates.Expressions import getEngine

# foreign product imports
from Products.OrderedFolder.OrderedFolder import OrderedFolder

# product imports

from Utility import intersect
from Utility import getTemplateFile
from Utility import initialize_permissions

class ActionDefContainer(OrderedFolder):
    security = ClassSecurityInfo()
    meta_types = ()
    icon = 'misc_/ReQuezt/ActionDefContainer'

    security.declareProtected(view, 'getValidActions')
    def getValidActions(self, **kw):
        user = getSecurityManager().getUser()
        roles = user.getRolesInContext(self)
        valid_actions = []
        for action in self.objectValues('ActionDef'):
            if action.valid(**kw):
                valid_actions.append(action)
        return valid_actions
    
InitializeClass(ActionDefContainer)
        
class ActionDef(PropertyManager, SimpleItem):
    """ provide information for methods (actions) """
    meta_type = "ActionDef"
    security = ClassSecurityInfo()
    icon = 'misc_/ReQuezt/ActionDef'
    _properties = PropertyManager._properties + \
                  ({'id': 'method_id', 'type': 'string', 'mode': 'w', 'default':''},
                   {'id': 'image_url', 'type': 'string', 'mode': 'w', 'default':''},
                   {'id': 'condition', 'type': 'lines', 'mode': 'w', 'default': []}
                   )
    manage_options =  PropertyManager.manage_options + SimpleItem.manage_options
    
    def __init__(self, id, title, method_id, image_url, condition):
        self.id = id
        for property in self._properties:
            if not hasattr(self, property['id']):
                setattr(self, property['id'], property['default'])
        self.manage_changeProperties(title = title,
                                     method_id = method_id,
                                     image_url = image_url,
                                     condition = condition)
        
        
    security.declareProtected(view, 'valid')
    def valid(self, **kw):
        user = getSecurityManager().getUser()
        if user.has_permission(view, self) and self.__call__(here=self, parent=self.aq_parent, options=kw):
            return 1
        else:
            return 0

    security.declarePublic('getMethodId')
    def getMethodId(self, **kw):
        if not hasattr(kw, 'here'):
            kw['here'] = self
        code = self.method_id
        if code:
            expr = getEngine().compile(code)
            return getEngine().getContext(kw).evaluate(expr)
        else:
            return ''
        
    def __call__(self, **kw):
        code = self.condition
        if ListType is type(code):
            code = " ".join(code).strip()
        if code:
            expr = getEngine().compile(code)
            return getEngine().getContext(kw).evaluate(expr)
        else:
            return 1

        
InitializeClass(ActionDef)

manage_addActionDefForm = getTemplateFile('manage_addActionDefForm', globals())
def manage_addActionDef(self, id, title, method_id, image_url, condition, roles, REQUEST=None):
    '''manage_addActionDef(self, id, method_id, image_url, condition, roles, REQUEST=None) -->
       add ActionDef to a ActionDefContainer;
       return None or HTML'''

    id = str(id)
    action = ActionDef(id, title, method_id, image_url, condition)
    self._setObject(action.id, action, set_owner=0)
    ob = self._getOb(action.id).__of__(self)
    initialize_permissions(ob, roles)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob

def manage_addActionDefContainer(self, id, title, REQUEST=None):
    id = str(id)
    ob = ActionDefContainer()
    ob.id = id
    ob.title = title
    self._setObject(id, ob)
    ob = self._getOb(id)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob
