# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

__version__='$Revision: 1.2 $'[11:-2]  
# python imports
import os, stat

# zope imports
import Globals
from AccessControl import ClassSecurityInfo
from OFS.Image import Image, cookId

# common products imports
from Products.PythonScripts.Utility import allow_class

# product imports
from Permissions import View

max_size = 2**23 # 8 MB Maximum Size for Attachments

def check_attachment_size(self, file, REQUEST=None):
    if hasattr(file, 'fileno'):
        mode = os.fstat(file.fileno())
        if mode[stat.ST_SIZE] > max_size:
            return 0
    return 1

class AttachmentTooLargeError(Exception):

    def __init__(self):
        Exception.__init__(self,"Attachment Too Large Error")
        
allow_class(AttachmentTooLargeError)

class Attachment(Image):  
    """ Attachment(Image)
        An Attachment. We use <Image> as base class, because <Image> provides
        image information for common image types, but also handles non image data.
    """  

    security = ClassSecurityInfo()
    meta_type = "Attachment"
    
    security.declarePrivate('__init__')
    def __init__(self,file=None, max_size=max_size):  
        if file:
            if not check_attachment_size(self, file):
                raise AttachmentTooLargeError
            id, title = cookId(None,None,file)
            Image.__init__(self,id,title,file)
  
    security.declareProtected(View, 'icon')
    def icon(self):
        return 'misc_/ReQuezt/attachment'

    security.declareProtected(View, 'filename')
    def filename(self):
        return self.getId()
  
    security.declareProtected(View, 'file_bytes')
    def file_bytes(self):  
        return self.get_size()
  
    security.declareProtected(View, 'file_kbytes')
    def file_kbytes(self):  
        return int(self.get_size()/1024)
  
Globals.InitializeClass(Attachment)
