# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python imports
from urllib import quote as url_quote

# zope imports
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from OFS.ObjectManager import BadRequestException
from OFS.ObjectManager import checkValidId
from AccessControl.Permissions import view, view_management_screens

# foreign product imports
from Products.OrderedFolder.OrderedFolder import OrderedFolder
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

# product imports

from Utility import getTemplateFile

LABEL_TYPE_TEXT = 'text'
LABEL_TYPE_IMG = 'img'
LABEL_TYPE_TEXT_AND_IMG='text_and_img'

class ColumnDefContainer(OrderedFolder):
    security = ClassSecurityInfo()
    meta_types = ()
    icon = 'misc_/ReQuezt/ColumnDefContainer'
    _properties = OrderedFolder._properties + \
                  ({'id': 'use_indexes_as_sort_keys', 'type': 'boolean', 'mode':'w', 'default': '1'},)
    
    def __init__(self, id, title, catalog_id):
        self.id = id
        self.title = title
        self.catalog_id = catalog_id
        for property in self._properties:
            if not hasattr(self, property['id']):
                setattr(self, property['id'], property['default'])

    def getValidSortKeys(self):
        catalog = getattr(self, self.catalog_id)
        keys = ['']
        if catalog:
            if self.use_indexes_as_sort_keys:
                keys.extend( catalog.Indexes.objectIds() )
            else:
                keys.extend( catalog._catalog.names )
        return keys
        
    def cookId(self, id):
        import traceback
        try:
            checkValidId(self, id)
        except BadRequestException:
            n = len(self.objectValues())
            id = id + "_%s" % n
            try:
                checkValidId(self, id)
            except BadRequestException:
                while 1:
                    id = "column_%s" % n
                    try:
                        checkValidId(self, id)
                    except BadRuestException:
                        n += 1
                    else:
                        break
        return id
    
    
InitializeClass(ColumnDefContainer)

class ColumnDef(ZopePageTemplate):
    """ provide information for methods (actions) """
    meta_type = "ColumnDef"
    security = ClassSecurityInfo()
    label_types = [LABEL_TYPE_TEXT, LABEL_TYPE_IMG, LABEL_TYPE_TEXT_AND_IMG]
    sort_directions = ['asc', 'desc']

    _properties = ZopePageTemplate._properties + \
                  ({'id': 'sort_key', 'type': 'selection', 'mode': 'w', 'default':'',
                    'select_variable':'getValidSortKeys'},
                   {'id': 'label_type', 'type': 'selection', 'mode': 'w', 'default':'',
                    'select_variable':'label_types'},
                   {'id': 'default_width', 'type': 'string', 'mode': 'w', 'default': ''}, 
#                   {'id': 'sort_method', 'type': 'string', 'mode': 'w', 'default': ''}, 
                   {'id': 'sort_dir', 'type': 'selection', 'mode': 'w', 'default': '',
                    'select_variable': 'sort_directions'},
                   {'id': 'align', 'type': 'string', 'mode': 'w', 'default': ''}, 
                   {'id': 'valign', 'type': 'string', 'mode': 'w', 'default': ''}, 
                   {'id': 'css_class', 'type': 'string', 'mode': 'w', 'default': ''}, 
                   {'id': 'style', 'type': 'string', 'mode': 'w', 'default': ''}
                   )
                  
    def __init__(self, id, sort_key, label_type, title, default_width, value):
        ZopePageTemplate.__init__(self, id, text=value)
        for property in self._properties:
            if not hasattr(self, property['id']):
                setattr(self, property['id'], property['default'])
        self.manage_changeProperties(sort_key=sort_key,
                                     label_type=label_type,
                                     title=title,
                                     default_width=default_width)

    def icon(self):
        if not self.sort_key or (self.sort_key in self.getValidSortKeys()):
            return 'misc_/ReQuezt/ColumnDef'
        else:
            return 'misc_/ReQuezt/ColumnDef_broken'
        
        
InitializeClass(ColumnDef)

manage_addColumnDefForm = getTemplateFile('manage_addColumnDefForm', globals())
def manage_addColumnDef(self, sort_key, label_type, title, default_width, value, REQUEST=None, **kw):
    '''manage_addColumnDef(self, id, title, label_type, sort_key, sort_method, default_width, REQUEST=None) -->
       add ColumnDef to a ColumnDefContainer;
       return None or HTML'''

    id = self.cookId(sort_key)
    col  = ColumnDef(id, sort_key, label_type, title, default_width, value)
    self._setObject(col.id, col)
    ob = self._getOb(col.id).__of__(self)
    ob.manage_changeProperties(REQUEST, **kw)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob
    

def manage_addColumnDefContainer(self, id, title='', catalog_id='', REQUEST=None):
    id = str(id)
    ob = ColumnDefContainer(id, title, catalog_id)
    self._setObject(id, ob)
    ob = self._getOb(id)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob
