# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

import locale
import string
import random
import re
from types import StringType, TupleType
word_sep = re.compile("\W+")
try:
    locale.setlocale(locale.LC_CTYPE, 'de')
except:
    pass

def remove_duplicates(sequence):
    l = list(sequence)
    l.sort()
    l = filter(None, map(lambda x, y: x != y and x,l, l[1:]))
    if TupleType is type(sequence):
        return tuple(l)
    else:
        return l

def sort_by_len(a, b):
    aS, aLen = a
    bS, bLen = b
    if aLen < bLen:
        return -1
    elif aLen == bLen:
        return 0
    else:
        return 1
        
class RandomDataGenerator:
    
    def __init__(self, wordlist_file=None):
        if wordlist_file is None:
            wordlist_file = '/usr/share/dict/words'
        if StringType is type(wordlist_file):
            wordlist_file = open(wordlist_file, 'r')
        wordlist = wordlist_file.readlines()
        wordlist = " ".join(wordlist)
        self.setWordlist(wordlist)

    def setWordlist(self, newWordlist):
        if StringType is type(newWordlist):
            newWordlist = word_sep.split(newWordlist)
        remove_duplicates(newWordlist)
        newWordlist = map(lambda x: (x, len(x)), newWordlist)
        newWordlist.sort(sort_by_len)
        smallword_idx = 0
        for word, length in newWordlist:
            smallword_idx += 1
            if length > 3: break
        self._wordlist = newWordlist
        self._smallword_idx = smallword_idx
        
    def getRandomText(self, avgLen=20):
        word_count = avgLen / 2 + random.randrange(1, avgLen)
        wordlist = self._wordlist
        wordlist_length = len(wordlist)
        smallword_idx = self._smallword_idx
        result = []
        for n in range(word_count):
            if random.randrange(1, 4) == 1:
                pos = random.randrange(1, smallword_idx)
                word, length = wordlist[pos]
            else:
                word, length = random.choice(wordlist)
            result.append(word)
        return " ".join(result)
    
    def getRandomWord(self):
        word, length = random.choice(self._wordlist)
        return word
    
    def getRandomString(self, avgLen=10, allowed_chars=string.letters):
        char_count = avgLen / 2 + random.randrange(1, avgLen)
        result = []
        for n in range(char_count):
            result.append(random.choice(allowed_chars))
        return "".join(result)
    
        
if __name__=='__main__':
    rg = RandomDataGenerator()
