# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python library imports
import os.path
from os.path import join as pjoin

# zope imports
from Globals import package_home
from AccessControl.Role import RoleManager
from AccessControl.SecurityInfo import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager
from OFS import Folder
try:
    from Products.StandardCacheManagers import RAMCacheManager
except ImportError:
    pass
from zLOG import LOG, ERROR, WARNING, BLATHER

# module imports
import ReQueztForum
import Skins
import ReQueztViewsFolder
import FormsFolder

from Utility import addMethod
from Utility import addImage
from Utility import addFile
from Utility import method_extensions
from Utility import image_extensions
from Utility import intersect
from Permissions import *
from Const import *

global_permissions = (('Add Documents, Images, and Files', (ManagerRole, ModeratorRole)),
                      ('Add ReQuezt Forums', (ManagerRole, ModeratorRole)),
                      ('Add Formulator Forms', (ManagerRole, ModeratorRole)),
                      ('Add MailHost objects', (ManagerRole, ModeratorRole)),
                      ('Add Ordered Folders', (ManagerRole, ModeratorRole)),
                      ('Add Page Templates', (ManagerRole, ModeratorRole)),
                      ('Change DTML Documents', (ManagerRole, ModeratorRole)),
                      ('Change DTML Methods', (ManagerRole, ModeratorRole)),
                      ('Change Formulator Fields', (ManagerRole, ModeratorRole)),
                      ('Change Formulator Forms', (ManagerRole, ModeratorRole)),
                      ('Change Images and Files', (ManagerRole, ModeratorRole)),
                      ('Change Page Templates', (ManagerRole, ModeratorRole)),
                      ('FTP access', (ManagerRole, ModeratorRole)),
                      ('Import/Export objects', (ManagerRole, ModeratorRole)),
                      ('Manage ZCatalog Entries', (ManagerRole, ModeratorRole)),
                      ('Manage properties', (ManagerRole, ModeratorRole)),
                      ('Undo changes', (ManagerRole, ModeratorRole)),
                      ('View management screens', (ManagerRole, ModeratorRole)),
                      ('WebDAV Lock items', (ManagerRole, ModeratorRole)),
                      ('WebDAV Unlock items', (ManagerRole, ModeratorRole)),
                      ('WebDAV access', (ManagerRole, ModeratorRole))
                      )
global_roles = (ModeratorRole,)

def createReQueztForum(self,
                       id,
                       title,
                       title_short,
                       mail_host,
                       exp,
                       moderated,
                       max_itemlist,
                       mail_entries_to_moderator,
                       mail_replies_to_moderator,
                       add_XUF,
                       add_moderator,
                       skin_id, 
                       create_default_docs,
                       ):
    ob = ReQueztForum.ReQueztForum(self, id, title, title_short, mail_host, skin_id, exp, moderated, max_itemlist,
                         mail_entries_to_moderator, mail_replies_to_moderator)
    ob = ob.__of__(self)
    initialize(ob, self, add_XUF, add_moderator, create_default_docs)
    return ob

def initialize(ob, container, add_XUF, add_moderator, create_default_docs):
    skin = Skins.getSkin(ob.skin_id)
    initialize_default_objects(ob, container, skin, create_default_docs)
    initialize_default_properties(ob, skin)
    if add_XUF in ['self', 'item']:
        xuf_container = ob
    elif add_XUF in ['parent', 'container']:
        xuf_container = container
    else:
        xuf_container = None
    if xuf_container:
        try:
            xuf_container.manage_addexUserFolder(skin.authSourceType,
                                                 skin.propSourceType,
                                                 skin.memberSourceType,
                                                 cookie_mode=1,
                                                 sessionTracking=1,
                                                 session_length=300,
                                                 not_session_length=300,
                                                 idleTimeout=None)
        except:
            import traceback
            LOG('ReQueztFactory',
                WARNING, "Couldn't add exUserFolder")
    if add_moderator:
        user = getSecurityManager().getUser()
        ob.manage_addLocalRoles(user.getUserName(), [ModeratorRole])
    initialize_permissions(ob)
    initialize_global_permissions(container._d) # evil hack! use container info from factory dispatcher
        


def initialize_default_properties(ob, skin):
    # Ngnnn.
    properties = ob._properties + skin._properties
    for property in properties:
        property_id = property['id']
        if not hasattr(ob, property_id):
            setattr(ob, property_id, property.get('default', None))
        else:
            ob._updateProperty(property_id, getattr(ob, property_id))

def initialize_permissions(ob, permission_defaults=None, acquire=0):
    # set default permissions
    permissions = permission_defaults or ob._permission_defaults
    for permission, roles in permissions:
        try:
            ob.manage_permission(permission, roles, acquire)
        except:
            import sys
            LOG(ob.meta_type,
                ERROR,
                'Error setting default roles',
                'Permission / Roles: %s/%s' % (permission, roles),
                error=sys.exc_info())
            
def initialize_global_permissions(container):
    userdefined_roles = container.userdefined_roles()
    create_roles = filter(lambda x, l=userdefined_roles: not x in l, global_roles)
    if create_roles:
        for role in create_roles:
            container._addRole(role)
        initialize_permissions(container, global_permissions, acquire=1)
    
        
def initialize_default_objects(ob, container, skin, create_default_docs=(None,)):
    """ create initial objects """
    # create special folder 'views' and an initial view
    if 'views' in create_default_docs:
        initialize_views(ob, skin)
    # create special folder 'forms' and import initial forms (and protect them from deletion)
    if 'forms' in create_default_docs:
        initialize_forms(ob, skin)
    # create special folder 'notifications' and import initial notifications (and protect them from deletion)
    if 'notifications' in create_default_docs:
        initialize_notifications(ob, skin)
    # create special folder 'menus' and import initial menus (and protect them from deletion)
    if 'menus' in create_default_docs:
        initialize_menus(ob, skin)
        
    # add default dtml methods (for class ReQuezt, a copy is added to each instance
    # we need them here to declare permissions)
    if 'templates' in create_default_docs:
        initialize_local_templates(ob, skin)

    if 'global_templates' in create_default_docs:
        add_global_templates(container, skin.getPath('global_templates'))
        add_resources(container, skin.getPath('res'))
    


    addColumnDefs(ob, skin)
    addActionDefs(ob, skin)
    
    add_ram_cache(ob)
    

##    ob._reserved_names = ob._reserved_names +  tuple(reserved_names)

def add_global_templates(container, template_path):
    import os
    template_path = os.path.normpath(template_path)
    path = os.path.join(package_home(globals()), template_path)
    for filename in os.listdir(path):
        name, ext = os.path.splitext(filename)
        if ext[1:] in method_extensions:
            try:
                addMethod(container, name, name, os.path.join(template_path, name))
            except:
#                import traceback
#                traceback.print_exc()
                LOG('ReQueztFactory',
                    WARNING,
                    "Coudn't add default template %s" % name,
                    "Sourcefile: %s/%s" % (path, filename))
                
def add_resources(container, resource_path):
    # unused. add resources, like images and javascript files, to a container
    import os
    path = os.path.join(package_home(globals()), resource_path)
    container.manage_addFolder('res', title='Resources')
    res_folder = container._getOb('res')
    files = os.listdir(path)
    for filename in os.listdir(path):
        if filename[0] == '.':
            continue
        if not os.path.isfile(os.path.join(path, filename)):
            continue
        
        name, ext = os.path.splitext(filename)
        if name[0] == '.':
            continue
        try:
            if ext[1:] in image_extensions:
                addImage(res_folder, filename, os.path.join(resource_path, filename))
            elif ext[2:] in method_extensions:
                addMethod(res_folder, name, name, os.path.join(resource_path, name))
            else:
                addFile(res_folder, filename, os.path.join(resource_path, filename))
        except Exception:
#            import traceback
#            traceback.print_exc()
            LOG('ReQueztFactory',
                WARNING,
                "Coudn't add default resource %s" % name,
                "Sourcefile: %s/%s" % (path, filename))

def initialize_views(ob, skin):
    views_folder_id = 'views'
    views_folder = ReQueztViewsFolder.ReQueztViewsFolder(__name__=views_folder_id)
    views_folder.id = views_folder_id
    ob._setObject(views_folder_id, views_folder)
    views_folder = ob._getOb(views_folder_id)
    for topic in skin.topics:
        id = topic.get('id')
        title = topic.get('title')
        title_short = topic.get('title_short')
        views_folder.manage_addReQueztTopic(id, title, title_short)
    for view  in skin.views:
        id = view.get('id')
        title = view.get('title')
        title_short = view.get('title_short')
        filter_method_file = view.get('filter_method')
        description_file = view.get('description')
        hidden = view.get('hidden')
        roles = view.get('roles')
        views_folder.manage_addReQueztView(id, title, title_short, hidden = hidden)
        view = views_folder._getOb(id)

        if roles:
            # add Owner to allowed roles, because some methods won't be accessible. Really.
            if not "Owner" in roles:
                roles = tuple(roles) + ("Owner",)
            view.manage_permission(permission_to_manage=View, roles=roles, acquire=0)
        if filter_method_file:
            filter_method_file = pjoin(skin.getPath('views'), filter_method_file)
            addMethod(view, 'getFilter', 'getFilter', filter_method_file, 'py')
        if description_file:
            description_file = pjoin(skin.getPath('views'), description_file)
            addMethod(view, 'description', 'Beschreibung', description_file)


    
def initialize_forms(ob, skin):
    forms_folder_id = 'forms'
    forms_folder = FormsFolder.FormsFolder(__name__=forms_folder_id)
    forms_folder.id = forms_folder_id
    ob._setObject(forms_folder_id, forms_folder)
    default_forms_ids = skin.forms_ids
    for form_name in default_forms_ids:
        filename = pjoin(skin.getPath('forms'), form_name)
        forms_folder.manage_form_import(filename)
        form = forms_folder[form_name]

def initialize_notifications(ob, skin):
    notifications_folder_id = 'notifications'
    notifications_folder = Folder.Folder(__name__=notifications_folder_id)
    notifications_folder.id = notifications_folder_id
    ob._setObject(notifications_folder_id, notifications_folder)
    for notification_name in skin.notifications_ids:
        filename = pjoin(skin.getPath('notifications'), notification_name)
        addMethod(notifications_folder, notification_name, notification_name, filename)

def initialize_menus(ob, skin):
    menus_folder_id = 'menus'
    menus_folder = Folder.Folder(__name__=menus_folder_id)
    menus_folder.id = menus_folder_id
    ob._setObject(menus_folder_id, menus_folder)
    default_menus_ids = []
    for menu in skin.menus:
        id = menu['id']
        title = menu['title']
        filename = pjoin(skin.getPath('menus'), id)
        roles = menu['roles']
        addMethod(menus_folder, id, title, filename)
        menu_obj = menus_folder[id]
        initialize_permissions(menu_obj, ((View, roles),), 0)
        default_menus_ids.append(id)

def initialize_local_templates(ob, skin):
    for template in skin.local_templates:
        id = template['id']
        title = template['title']
        filename = pjoin(skin.getPath('local_templates'), id)
        roles = template['roles']
        addMethod(ob, id, title, filename)
        templ_obj = getattr(ob, id)
        initialize_permissions(templ_obj, ((View, roles),), 0)

def add_ram_cache(ob):
    # try to add a ramcachemanager and associate 'index_html.pt' with it. emit a warning if this fails
    try:
        ramcache_id = 'ramcache'
        ramcache = RAMCacheManager.RAMCacheManager(ramcache_id)
        ramcache_settings = ramcache.getSettings()
        ramcache_settings['request_vars'] = ()
        ramcache.manage_editProps(ramcache_id, ramcache_settings)
        ob._setObject(ramcache_id, ramcache)
        template = getattr(ob, 'index_html.pt')
        template.ZCacheable_setManagerId(ramcache_id)
        template.ZCacheable_setEnabled(enabled=1)
    except:
        import traceback
        LOG('ReQueztFactory',
            WARNING,
            'Error adding RAMCacheManager for %s' % ob.getId(),
            traceback.format_exception(*traceback.sys.exc_info()))

def addColumnDefs(ob, skin):
    skin.addColumnDefs(ob, 'column_defs', catalog_id=ob.id)

def addActionDefs(ob, skin):
    skin.addActionDefs(ob, 'action_defs')
