# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#
__doc__ = """ReQuezt Forum"""
__version__='$Revision: 1.3 $'[11:-2]

# python core imports
import sys
from time import time
from types import *

# zope core imports
from zLOG import LOG, ERROR, WARNING, INFO, BLATHER, DEBUG
from Acquisition import aq_base
from BTrees.IOBTree import IOBTree
from intSet import intSet
from AccessControl import ClassSecurityInfo
from AccessControl import SecurityManagement
from AccessControl.unauthorized import Unauthorized
from ZTUtils import Batch
from DocumentTemplate import sequence

import Globals
from Globals import MessageDialog
from Globals import InitializeClass

# zope content class imports
from OFS.Folder import Folder
from OFS.ObjectManager import REPLACEABLE
from Products.ZCatalog import ZCatalog

# third party Products imports
from Products.OrderedFolder.SubObjectTypeSupport import SubObjectTypeSupport
from Products.Formulator.Form import FormValidationError

# ReQuezt Forum module imports
from Utility import getitem
from Utility import getTemplateFile
from Utility import remove_duplicates
from Utility import url_quote
from Utility import url_unquote

# ReQuezt Forum class imports
import ReQueztFactory
import ReQueztView
import FormRenderer
import Entries
import Attachment
import Skins
from UserDataWrapper import createUserDataWrapper
from Attachment import check_attachment_size
from Const import *
from Permissions import *

# constructors
manage_addReQueztForm = getTemplateFile('manage_addReQueztForm', globals())

def manage_addReQuezt(self,
                      id,
                      title='',
                      title_short='',
                      mail_host=None,
                      exp=None,
                      moderated='none',
                      max_itemlist=DEFAULT_MAX_ITEMLIST,
                      mail_entries_to_moderator=0,
                      mail_replies_to_moderator=0,
                      add_XUF='',
                      add_moderator=0,
                      skin_id='',
                      create_default_docs=(None,),
                      REQUEST=None):
    '''manage_addReQuezt(self, id, title='', mail_host=None,
    default_doc=0, moderated='none', max_itemlist=20, add_moderator=0,
    mail_entries_to_moderator=0, mail_replies_to_moderator=0, add_XUF, REQUEST=None) -->
    add new ReQuezt;
    return None or HTML'''

    ob = ReQueztFactory.createReQueztForum(self, id,
                                           title,
                                           title_short,
                                           mail_host,
                                           exp,
                                           moderated,
                                           max_itemlist,
                                           mail_entries_to_moderator,
                                           mail_replies_to_moderator,
                                           add_XUF,
                                           add_moderator,
                                           skin_id,
                                           create_default_docs)
    self._setObject(ob.getId(), ob)

    if REQUEST:
        return self.manage_main(self, REQUEST, update_menu=1)

# main class
class ReQueztForum(SubObjectTypeSupport, FormRenderer.FormRenderer, ZCatalog.ZCatalog):
    """ class ReQueztForum(SubObjectTypeSupport, ZCatalog.ZCatalog):
        ReQuezt application main class. Provides container for messages, helper
        methods for form processing and user management.
    """
    meta_type  ='ReQuezt Forum'
    description='ReQuezt Forum'
    icon       ='misc_/ReQuezt/ReQueztForum'
    selected_meta_types = ('DTML Method', 'DTML Document', 'Page Template',
                           'Folder', 'Ordered Folder',
                           'Image', 'Script (Python)', 'File')

    # set up security for ReQueztForum
    __ac_roles__ = ZCatalog.ZCatalog.inheritedAttribute('__ac_roles__') + (ExpertRole, ModeratorRole)
    
    security = ClassSecurityInfo()

    _permission_defaults = ((ManageReQueztForum,    (ManagerRole, ModeratorRole)),
                            (ModerateEntries,   (ManagerRole, ModeratorRole)),
                            (AddEntries,        (ManagerRole, ExpertRole, AnonymousRole)),
                            (AddReplies,        (ManagerRole, ModeratorRole, ExpertRole)),
                            (AddComments,       (ManagerRole, ModeratorRole)),
                            (EditEntries,       (ManagerRole, ExpertRole, ModeratorRole)),
                            (View,              (ManagerRole, AnonymousRole)),
                            (ViewUserInfo,      (ManagerRole, AuthenticatedRole))
                            )

    for permission, roles in _permission_defaults:
        security.setPermissionDefault(permission, roles)
    # as of now, permissions also get set on object creation within ReQueztFactory,
    # because of a bug in setPermissionDefault

    _properties               = ({'id':'title', 'type':'string','mode':'w', 'default': ''},
                                 {'id':'title_short', 'type':'string', 'mode': 'w', 'default': ''} ,
                                 {'id':'max_wait', 'type':'int', 'mode': 'w', 'default': 72} ,
                                 {'id':'forum_email_address', 'type': 'string', 'mode': 'w', 'default': ''},
                                 {'id':'mail_entries_to_moderator', 'type': 'boolean', 'mode': 'w', 'default': 1},
                                 {'id':'mail_replies_to_moderator', 'type': 'boolean', 'mode': 'w', 'default': 0}, 
                                 {'id':'expire', 'type': 'boolean', 'mode': 'w', 'default': 0}, 
                                 {'id':'moderated', 'type': 'boolean', 'mode': 'w', 'default': 0},
                                 {'id':'mod_comment', 'type': 'boolean', 'mode': 'w', 'default': 0}, 
                                 {'id':'mod_reply', 'type': 'boolean', 'mode': 'w', 'default': 0},
                                 {'id':'max_itemlist', 'type': 'int', 'mode': 'w', 'default': 20}
                                 )

    # Defaults
    root = 1
    level = -1
    thread = []
    _vocabulary_id = "Vocabulary"

    # management tabs
    manage_options = ({'label':'Contents', 'icon':icon, 'action':'manage_main', 'target':'manage_main'},
                      {'label':'View', 'icon':'', 'action':'index_html', 'target':'manage_main'},
                      {'label':'Entries', 'icon':'', 'action':'manage_entries', 'target':'manage_main'},
                      {'label':'Topics &amp; Views', 'icon':'', 'action':'views/manage_main', 'target':'manage_main'},
                      {'label':'Options', 'icon':'', 'action':'manage_edit_form', 'target':'manage_main'},
                      {'label':'Properties', 'icon':'', 'action':'manage_propertiesForm', 'target':'manage_main'},
                      {'label':'Users', 'icon':'', 'action':'manage_edit_users_form', 'target':'manage_main'}) \
                      + SubObjectTypeSupport.manage_options \
                      + ({'label':'Security', 'icon':'', 'action':'manage_access', 'target':'manage_main'},
                         {'label':'Undo', 'icon':'', 'action':'manage_UndoForm', 'target':'manage_main'}
                         )

    # management forms
    security.declareProtected(ModerateEntries, 'manage_entries')
    manage_entries = getTemplateFile('templates/ReQueztForum_manage_entries', globals())

    security.declareProtected(ManageReQueztForum, 'manage_edit_form')
    manage_edit_form = getTemplateFile('templates/ReQueztForum_edit_form', globals())

    security.declareProtected(ManageReQueztForum, 'manage_edit_users_form')
    manage_edit_users_form = getTemplateFile('templates/ReQueztForum_edit_users_form', globals())

    security.declarePrivate('_manage_edit_user_data_form')
    _manage_edit_user_data_form = getTemplateFile('templates/ReQueztForum_edit_user_data_form', globals())

    security.declarePrivate('_manage_edit_user_data_form_body')
    _manage_edit_user_data_form_body = getTemplateFile('templates/ReQueztForum_edit_user_data_form_body', globals())

    security.declarePrivate('_ui_edit_user_data_form')
    _ui_edit_user_data_form = getTemplateFile('templates/ReQueztForum_ui_edit_user_data_form', globals())

    security.declarePrivate('_buildIndexing')
    def _buildIndexing(self, id, title):
        # Initialise ZCatalog
        if not hasattr(self, self._vocabulary_id):
            from Products.PluginIndexes.TextIndex import Splitter
            from Products.PluginIndexes.TextIndex import Vocabulary
            splitter = Splitter.availableSplitters[0]
            for splitter_id, splitter_name  in Splitter.availableSplitters:
                if splitter_id.upper().find("ISO_8859_1") >= 0:
                    splitter = splitter_id
                    break
            Vocabulary.manage_addVocabulary(self, self._vocabulary_id, splitter,
                                            globbing=1, splitter=splitter_id, REQUEST=None)
        if not hasattr(self,'_catalog'):

            ZCatalog.ZCatalog.__init__(self, id, title, self._vocabulary_id, self)

        base = aq_base(self)
        addIndex = self.addIndex
        addColumn = self.addColumn

        # Set up the indexes
        # We get a 'title' TextIndex for free from ZCatalog.ZCatalog
        for (name,index_type) in self.getOption('indexes'):
            try:
                addIndex(name,index_type)
            except 'Index Exists':
                pass

        # Add the meta data columns for search results
        for name in self.getOption('meta_data_columns'):
            try:
                addColumn(name,'')
            except 'Column Exists':
                pass
            except:
	        pass
#                print sys.exc_info()

        # set default index titles
        self.index_titles = {}



    security.declareProtected(ManageReQueztForum, 'updateIndexing')
    def updateIndexing(self,REQUEST=None):
        """ A a ZCatalog and appropriate indexes to this ReQuezt Forum if they aren't there already """
        self._buildIndexing(self.getId(), self.title)
        self.recatalogentries(REQUEST)

    security.declareProtected(ModerateEntries, 'recatalogentries')
    def recatalogentries(self,REQUEST=None):
        """ Clear the Catalog and then Index all the entries. """
        self._catalog.clear()
        for id in self._ids:
            entry = self.get_wrapped_entry(id)
            entry.index(reindex_parent=0)
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

    # sort out the catalog if we're moved, copied or renamed
    security.declareProtected(ManageReQueztForum, 'manage_afterAdd')
    def manage_afterAdd(self, item, container):
        try:
            self.recatalogentries()
        except AttributeError:
            self.updateIndexing()

    security.declarePrivate('__init__')
    def __init__(self,
                 parent,
                 id,
                 title,
                 title_short,
                 mail_host,
                 skin_id,
                 exp,
                 moderated,
                 max_itemlist,
                 mail_entries_to_moderator,
                 mail_replies_to_moderator):
        self.data = IOBTree()  # id -> Message
        self._ids = intSet() # ids of children
        t = time()
        self.id = id
        title = title or id
        self.title = title
        self.title_short = title_short or id
        self.created = t
        self.modified = t
        self.mail_host = mail_host
        self.skin_id = skin_id # we'd rather not use this. We'd rather have a 'skin' folder or use CMF skinning

        # don't know if we want to keep this 'moderation' stuff.
        if moderated == 'both':
            moderated = 1
            mod_reply = 1
        elif moderated == 'entries':
                moderated = 1
                mod_reply = 0
        else:
            moderated = 0
            mod_reply = 0
            
        self.manage_changeProperties(expire=exp, mail_entries_to_moderator = mail_entries_to_moderator,
                                     mail_replies_to_moderator = mail_replies_to_moderator,
                                     moderated = moderated, mod_reply = mod_reply,
                                     max_itemlist = max_itemlist)
        
        self._buildIndexing(id,title)

        
    security.declarePublic('__len__')
    def __len__(self): return 1

    security.declarePrivate('__getitem__')
    __getitem__ = getitem
    
    security.declarePrivate('get_wrapped_entry')
    def get_wrapped_entry(self,id):
        # returns a entry wrapped as if it had been traversed to
        obj = self.data[id]
        return self.unrestrictedTraverse(self.getPhysicalPath()+tuple(map(str,obj.getThread())) + (obj.getId(),))

    security.declareProtected(ManageReQueztForum, 'getOption')
    def getOption(self, id):
        skin = getattr(self, '_v_skin', Skins.getSkin(self.skin_id))
        return skin.getOption(id)
    
    security.declareProtected(View, 'topics_list')
    def topics_list(self, include_names = 1, ids=None):
        # ''' lists all topics '''
        return self.views_list(include_names=include_names, meta_types=[ReQueztView.ReQueztTopic.meta_type], ids=ids)
    
    security.declareProtected(View, 'views_list')
    def views_list(self, include_names = 1, meta_types = None, ids=None):
        # ''' lists all views '''
        retval = []
        views = self.views.objectValues(meta_types)
        if ids is not None:
            views = filter(lambda x, ids=ids: x.getId() in ids, views)
        for view in views:
            if include_names:
                retval.append((view.title_or_id(), view.getId()))
            else:
                retval.append(view.getId())
        return retval

    security.declarePrivate('setItem')
    def setItem(self,id,obj,index=1):
        # Make sure the object we store is not wrapped with
        # an acquisition wrapper, since we will be wrapping
        # it again manually in __getitem__ when it is needed.
        bobj = getattr(obj,'aq_base',obj)
        self._ids.insert(id)
        self.data[id]=bobj

        # index the entry
        if index:
            obj.index()

        #invalidate cached index_html
        self._touch()

    security.declarePrivate('delItem')
    def delItem(self,id):
        physical_path = self.getPhysicalPath()
        # get the item to delete
        data=self.data
        try:
            item=data[id].__of__(self)
        except KeyError:
            LOG('ReQuezt Forum',
                DEBUG,
                'Error deleting item',
                'Forum: %s, id to be deleted: %s' % (self.absolute_url(), id),
                error=sys.exc_info())
            return 0

        # make a list of ids to be deleted
        # (the id specified, and all it's child entries)
        ids=intSet()
        ids.insert(id)
        ids=item.sub_ids(ids)

        review_count = item._revsub

        if not item._reviewed:
            review_count += 1

        if self._ids.has_key(id):
            self._ids.remove(id)

        # ...otherwise, things are a bit more complicated
        else:
            parent = data[item.getThread(-1)]

            # remove it from it's parents list of ids
            parent._ids.remove(id)

            for i in item.getThread():
                entry = data[i]

                # adjust the count of unreviewed replies for each entry in the thread
                entry._revsub = entry._revsub - review_count

            # re-index the parent, so counts get reset
            self.get_wrapped_entry(item.getThread(-1)).index()

        item=None

        # Un-catalog the objects to be deleted.
        for id in ids:
            obj = data[id].__of__(self)
            oid = "%s%s/%s" % ('/'.join(self.getPhysicalPath()), obj.thread_path(), id)
            self.uncatalog_object(oid)

        for id in ids:
            del data[id]

        self._touch()

    security.declarePrivate('_touch')
    def _touch(self):
        ''' call this when object has changed to notify other objects, e.g. to invalidate caches or whatever. '''
        self.index_html__ZCacheable_invalidate()

    security.declarePrivate('createId')
    def createId(self):
        id=int(time())
        while self.data.has_key(id):
            id += 1
        return id

    security.declarePrivate('data_map')
    def data_map(self,ids):
        result=[]
        for id in ids:
            result.append(self.data[id].__of__(self))
        return result

    security.declareProtected(View, 'entryValues')
    def entryValues(self):
        # """ return list of entries """
        entries = self.data_map(self._ids)
        entries.reverse()
        return entries

    security.declareProtected(View, 'tpId')
    def tpId(self):
        return self.id

    security.declareProtected(View, 'tpURL')
    def tpURL(self):
        return self.id

    security.declareProtected(View, 'this')
    def this(self):
        return self

    security.declareProtected(View, 'requezt_base')
    def requezt_base(self):
        return self

    security.declareProtected(View, 'base_url')
    def base_url(self):
        # """ url of the main page """
        return self.absolute_url()

    security.declareProtected(View, 'has_items')
    def has_items(self):
        return len(self._ids)

    security.declareProtected(ModerateEntries, 'item_count')
    def item_count(self):
        return len(self.data)

    security.declareProtected(AddReQueztForums, 'mailhost_list')
    def mailhost_list(self):
        #  """ list of mail hosts """
         try:    return self.superValues(('Mail Host',))
         except: return []

    security.declarePrivate('expire_items')
    def expire_items(self):
        """ run to delete entries which are past their expiration (assuming that it has been set)"""
        if self.expire:
            d=self.data
            t=int(time()-(self.expire * 86400.0))
            ids=[]
            for id in d.keys():
                if d[id].modified < t:
                    ids.append(id)
                for id in ids:
                    try:    self.delItem(id)
                    except: pass
        return ''

########
## methods related to adding new entries. Also see FormRenderer
########
    
    security.declareProtected(AddEntries, 'suggest_author')
    def suggest_author(self):
        author = ''
        try: author = self.REQUEST.cookies['_suggest_author']
        except: pass
        if author: author = url_unquote(author)
        return author

    security.declareProtected(AddEntries, 'suggest_email')
    def suggest_email (self):
        email = self.REQUEST.cookies.get('_suggest_email', '')
        if email: email = url_unquote(email)
        return email

    security.declareProtected(AddEntries, 'suggest_notify')
    def suggest_notify (self):
        notify = self.REQUEST.cookies.get('_suggest_notify', 0)
        return notify

    security.declareProtected(AddEntries, 'addEntry')
    def addEntry(self,form_id, REQUEST=None,RESPONSE=None, index=1):
        """ add an entry """
        # simple locking mechanims to prevent repeated POSTs
        session_data = self.getSessionData()
        if REQUEST is not None:
            is_processing = session_data.has_key('_locked')
            if is_processing:
                if (time() - session_data['_locked']) > 60:
                    # might be a stale lock
                    del session_data['_locked']
                RESPONSE.redirect(self.absolute_url())
                return
            else:
                session_data['_locked'] = time()

        # form validation / entry creation
        try:
            # also adds entry to self.data (calls setItem)
            entry=Entries.addItem(self, Entries.Entry, form_id, index, REQUEST, RESPONSE)
        except FormValidationError, error_value:
            if session_data.has_key('_locked'):
                del session_data['_locked']
            REQUEST['error_value'] = error_value
            return self.add_entry_form(self, REQUEST=REQUEST)
        topic = entry.topic
        expert = entry.expert

        message = messages[ENTRY_ADDED]


        # streamwrite a message to response to avoid latency
        if REQUEST is not None:
            resp=REQUEST.RESPONSE
            p2 = "%s/index_html" % self.absolute_url()
            show_message = self.show_message(self, REQUEST=REQUEST, title=message.title,
                                     message=message.text,
                                     action=p2,
                                     params={'topic':topic})
            resp.setHeader('content-type', 'text/html')
            resp.setHeader('content-length', len(show_message))
            resp.write(str(show_message))

        # self.expire_items()

        # send notification emails
        if self.mail_entries_to_moderator:
            email = self.get_forum_email_address()
            self.send_email(self, entry=entry,event_id=ENTRY_ADDED,user_id = expert, email=email,REQUEST=REQUEST)
        if REQUEST is None: # not a web request-> return freshly created obj
            return entry

########
## manage session data and provide information for customized views
########
        
    security.declareProtected(ACCESS_SESSIONDATA_PERM, 'getSessionData')
    def getSessionData(self):
        request = self.REQUEST
        session = request.SESSION
        session_data_key = self.absolute_url()
        if not session.has_key(session_data_key):
            session[session_data_key] =  {}
        session_data = session.get(session_data_key)
        return session_data

    security.declarePrivate('aggregateIndex')
    def aggregateIndex(self, REQUEST):
        ''' generates a unique index for REQUEST '''
        session_data = self.getSessionData()
        index = {'forum_id': self.getId(),
                 'start': REQUEST.get('start') or session_data.get('start'),
                 'view': self.get_active_view_id(),
                 'sort_on': str(self.get_sort_descriptor(REQUEST=REQUEST,
                                                         update=1)),
                 'max_itemlist': REQUEST.get('max_itemlist',
                                             self.max_itemlist)
                       }
        index.update(self.getFilter())
        return index

    security.declareProtected(View, 'getCatalogIndexIds')
    def getCatalogIndexIds(self):
        return self.Indexes.objectIds()

    security.declareProtected(View, 'getCatalogIndexTitle')
    def getCatalogIndexTitle(self, index_id):
        titles = self.getOption('index_titles')
        if getattr(self, 'index_titles', None):
            titles.update(self.index_titles)
        if titles.has_key(index_id) and titles[index_id]:
            return titles[index_id]
        elif hasattr(self.Indexes, index_id):
            return index_id
        else:
            return 'Unknown Index'

    security.declareProtected(View, 'getMasterTemplate')
    def getMasterTemplate(self):
        master_template_id = self.getOption('master_template_id')
        master_template = getattr(self, master_template_id)
        return master_template.aq_base
    
    security.declareProtected(View, 'index_html')
    def index_html(self, REQUEST=None):
        """ we wrap this around index_html.pt for two reasons:
            1) keep some logic out of the template
            2) call template with what we know are unique keys for this
               request and pass anything else in the request.
               enables caching (:
        """
        request = REQUEST or self.REQUEST
        session_data = self.getSessionData()
        
        if session_data.has_key('_locked'):
            del session_data['_locked']

        template = getattr(self, 'index_html.pt')
        unique_key = self.aggregateIndex(REQUEST)
        # we'll have to do this here because else our previous/next buttons won't work
        # with caching enabled
        if session_data.get('_result_key') != unique_key:
            result_set = self.getResultSet()
            session_data['_result_key'] = unique_key
            # prevent result_set from being called twice
            # template tries request first and then calls getResultSet
            request.set('result_set', result_set)
        return template(**unique_key)

    security.declarePrivate('index_html__ZCacheable_invalidate')
    def index_html__ZCacheable_invalidate(self):
        template = getattr(self, 'index_html.pt')
        try:
            template.ZCacheable_invalidate()
        except:
	     pass
#            import traceback
#            traceback.print_exc()

    security.declarePrivate('getCurrentResultIds')
    def getCurrentResultIds(self):
        session_data = self.getSessionData()
        current_result_ids = session_data.get('current_result_ids', None)
        if current_result_ids is None:
            result_set = self.getResultSet()
            current_result_ids = result_set['current_result_ids']
        return current_result_ids
        
    security.declareProtected(View, 'getResultSet')
    def getResultSet(self, REQUEST=None):
        ''' index_html(self, REQUEST=None):
        get a ZTUtils.Batch according to the current filter, sort it
        and render the index_html.pt template using the results '''
        # use catalog sorting -> faster
        
        request = REQUEST or self.REQUEST
        session_data = self.getSessionData()
        use_catalog_sort = (self.column_defs.use_indexes_as_sort_keys)
            
        base = self.requezt_base()
        start = int(request.get('start', session_data.get('start', 0)))
        session_data['start'] = start
        size = int(base.max_itemlist)
        active_view = self.get_active_view_id()
        filter = base.getFilter()
        sort_on = base.get_sort_descriptor()
        if use_catalog_sort:
            # use internal catalog sorting -> much faster
            sort_key, sort_method, sort_dir = sort_on[0]
            reverse = (sort_dir == 'desc') and 'reverse' or None
            if reverse:
                filter['sort-order'] = 'reverse'
            filter['sort-on'] = sort_key
        results = base(filter)
        result_length = len(results)
        if use_catalog_sort:
            sorted_results = results
        else:
            sorted_results = sequence.sort(results, sort_on)
        entries = Batch(sorted_results, size, start, orphan=size/5)
        # remember the list of ids for this resultset.
        # (Remember: it's a catalog searchResult of mybrains instances, you can't pickle this. Really!)
        current_result_ids = map(lambda x:int(x.id), sorted_results)
        session_data['current_result_ids'] = current_result_ids

        return {'entries':entries, 'active_view': active_view,
                'sort_on': sort_on, 'result_length': result_length,
                'current_result_ids': current_result_ids
                }

    security.declareProtected(View, 'getFilter')
    def getFilter(self, REQUEST=None):
        # ''' get_entry_filter(self, REQUEST):
        RESET = 'reset'
        REPLACE = 'replace'
        UPDATE = 'update'
        REQUEST = REQUEST or self.REQUEST

        # get current filter from session data
        session_data = self.getSessionData()
        filter_from_session = session_data.get('filter', None)
        curr_flt = filter_from_session
        if curr_flt is None:
            curr_flt = self.views.default.getFilter()
        new_flt = None

        # get update mode from request
        flt_upd = REQUEST.get('flt_upd', RESET)

        update = (flt_upd == UPDATE)

        # try to get new filter data from formdata in REQUEST
        if REQUEST.has_key('form_id'):
            form_id = REQUEST.get('form_id', None)
            if form_id:
                form = getattr(self.forms, form_id, None)
                if form:
                    new_flt = form.validate_all(REQUEST)
                    # We don't catch FormValidationErrors here!
                    # Catch em above (caller)!
        else: # try to get new filter data from REQUEST
            new_flt = REQUEST.get('filter', None)

        if new_flt is not None:
            if flt_upd == RESET:
                default_filter = self.views.default.getFilter()
                default_filter.update(new_flt)
                new_flt = default_filter
            if update:
                curr_flt.update(new_flt)
                new_flt = curr_flt
        else:
            new_flt = curr_flt


        # update REQUEST so it reflects the current filter settings.
        idx_ids = self.getCatalogIndexIds()
        for key, value in new_flt.items():
            usage = (key[-6:] == "_usage")
            if not usage and key not in idx_ids:
                del new_flt[key]
            else: # try to convert to integer for numeric indexvalues
                try:
                    new_flt[key] = int(value)
                except: pass

        for key in idx_ids:
            if new_flt.has_key(key):
                REQUEST[key] = new_flt[key]
            elif not update:
                if REQUEST.form.has_key(key):
                    del REQUEST.form[key]
                if REQUEST.other.has_key(key):
                    del REQUEST.other[key]

        session_data['filter'] = new_flt
        return new_flt

    security.declareProtected(View, 'get_active_view_id')
    def get_active_view_id(self):
        # return the id of the currently used view. Default to views.default

        session_data = self.getSessionData()
        active_view = session_data.get('active_view', None)
        if active_view is None:
            active_view = DEFAULT_VIEW_ID
            session_data['active_view'] = active_view
        return active_view

    security.declareProtected(View, 'get_active_view')
    def get_active_view(self):
        view_id = self.get_active_view_id()
        return self.views[view_id]

    def get_help_context(self):
        request = self.REQUEST
        try:
            published = request['PUBLISHED']
            if hasattr(published, 'id'):
                id = published.id
            else:
                id = published.__name__
            all_contexts = self.getOption('help_contexts')
            view_id = self.get_active_view_id()
            view_contexts = all_contexts.get(view_id, {})
            default_contexts = all_contexts.get(DEFAULT_VIEW_ID, {})
            context = view_contexts.get(id, default_contexts.get(id, ''))
        except:
#            import traceback
#            traceback.print_exc()
            context = ''
        return context

    security.declareProtected(View, 'get_sort_descriptor')
    def get_sort_descriptor(self, REQUEST=None, update=0):
        # get a descriptor (tuple of tuples(key, method, direction)
        # for the sequence.sort method
        # ! returns None, if 'sort_key' is not present in REQUEST
        def get_sort_direction(new_sort_direction, curr_sort_direction=None,
                               valid_directions = ['asc', 'desc']):
            if new_sort_direction in valid_directions and new_sort_direction != curr_sort_direction:
                return new_sort_direction
            elif curr_sort_direction in valid_directions and new_sort_direction:
                return valid_directions[len(valid_directions) - valid_directions.index(curr_sort_direction) - 1]
            elif curr_sort_direction:
                return curr_sort_direction
            else:
                return valid_directions[0]

        session_data = self.getSessionData()
        get_option = self.getOption
        if update:
            new_sort_key = REQUEST.get('sort_key', None)
            new_sort_method = REQUEST.get('sort_method', None)
            new_sort_direction = REQUEST.get('sort_direction', None)
            if new_sort_key:
                curr_sort_desc = session_data.get('sort_on')
                if curr_sort_desc:
                    curr_sort_key, curr_sort_method, curr_sort_direction = curr_sort_desc[0]
                    flip = (new_sort_key != curr_sort_key)
                    new_sort_direction = get_sort_direction(new_sort_direction, curr_sort_direction)
                else:
                    new_sort_direction = get_sort_direction(new_sort_direction)
                new_sort_method = new_sort_method or 'cmp'
##                 descriptor = ((new_sort_key, new_sort_method, new_sort_direction),
##                               get_option('secondary_sort_descriptor'))
                descriptor = ((new_sort_key, new_sort_method, new_sort_direction),)
            else:
##                 descriptor = session_data.get('sort_on',
##                                               (get_option('sort_descriptor'),
##                                                get_option('secondary_sort_descriptor'))
##                                               )
                descriptor = session_data.get('sort_on',
                                              (get_option('sort_descriptor'),)
                                              )
            session_data['sort_on'] = descriptor
        else:
##             descriptor = session_data.get('sort_on',
##                                           (get_option('sort_descriptor'),
##                                            get_option('secondary_sort_descriptor'))
##                                            )
            descriptor = session_data.get('sort_on',
                                          (get_option('sort_descriptor'),)
                                           )                                      
        return descriptor

    security.declarePublic('get_sort_param')
    def get_sort_param(self, sort_key, sort_descriptor, sort_direction):
        ''' return a query string for the given sort descriptor '''
        params = []
        for param in (sort_key, sort_descriptor, sort_direction):
            params.append(param and str(param) or '')
        params = map(url_quote, params)
        params = map(None, ('sort_key', 'sort_method', 'sort_direction'), params)
        params = filter(lambda x: x[1], params)
        params = map('='.join, params)
        return "&".join(params)



# Searchable interface.

    security.declareProtected(View, '__call__')
    def __call__(self, REQUEST=None, internal=0, **kw):
        brains = apply(self.searchResults,(REQUEST,),kw)
        if internal:
            return map(lambda x: x.getObject(), brains)
        return brains

    security.declareProtected(View, 'show_message')
    show_message = getTemplateFile("templates/show_message", globals())
    show_message.__replaceable__ = REPLACEABLE

########
## generating and sending email notifications
########
    
    security.declareProtected(View, 'get_forum_email_address')
    def get_forum_email_address(self):
        forum_email_address = getattr(self, 'forum_email_address', '')
        if not forum_email_address:
            moderators = self.moderators_list(include_names=0)
            if moderators:
                udm = self.get_user_data_manager()
                moderator_email = udm.get_user_email(moderators[0])
                forum_email_address = moderator_email
            else:
                forum_email_address = self.getOption('fallback_email_address')
        return forum_email_address

    security.declarePublic('check_attachment_size')
    check_attachment_size = check_attachment_size

    security.declarePublic('validate_notification')
    def validate_notification(self, notify, REQUEST):
        form_id = REQUEST.get('form_id', 'frmEntry')
        email = self.forms[form_id].get_field('fldEmail').validate(REQUEST)
        if notify and not email:
            return 0
        else:
            return 1

    security.declarePrivate('send_email')
    def send_email(self, client, entry, event_id, REQUEST, email=None, user_id=None, **kw):
        # sends an email to the address using the mailhost, if there is one
        udm = self.get_user_data_manager()
        try:
            if self.mail_host:
                mhost=getattr(self,self.mail_host)
                # collect email addresses
                if user_id:
                    if ListType is type(user_id):
                        _email = map(udm.get_user_email,user_id)
                    else:
                        _email = [udm.get_user_email(user_id)]
                else:
                    _email = []
                if email:
                    if ListType is type(email):
                        _email = email + _email
                    else:
                        _email.append(email)

                email = filter(None, _email)
                email = remove_duplicates(email)
                recipient_addresses = ", ".join(email)
                sender_addresses = self.get_forum_email_address()
                mail =self.mail_html(client, newItem=entry,
                                     event_id=event_id,
                                     recipient_addresses=recipient_addresses,
                                     sender_addresses=sender_addresses, REQUEST=REQUEST, **kw)
                mhost.send(mail, recipient_addresses, sender_addresses)
                return 1
        except:
            # something bad happened :-(
            LOG('ReQuezt Forum',
                ERROR,
                'Error sending notification email',
                'URL: %s%s/%s\nAddresses: %s' % (self.absolute_url(),entry.thread_path(),entry.getId(), email),
                error=sys.exc_info())
            return 0

    def mail_html(self, client, newItem=None, event_id = None, recipient_addresses=None, sender_addresses=None,  REQUEST=None, **kw):
        """ mail_html(self, client, REQUEST, newItem=None, event_id=None, email=None, **kw):
            return an email-messages as notification for the event given in evend_id. newItem
            an AbstractEntry or subclass thereof. email contains the "To:" address string for
            the notification-email. Optional keyword arguments can be provided (the method rendering
            email messages must not fail only newItem, event_id and email is given, but may give extra
            info when optional keyword arguments are given) """
        notifications_fld = self.notifications
        method = getattr(notifications_fld, event_id, notifications_fld.default)
        return method(client, newItem=newItem,
                      evend_id=event_id,
                      REQUEST=REQUEST, **kw)

######
## user data and user management related stuff
######

    
    security.declarePublic('login')
    def login(self, REQUEST=None):
        """ login as privileged user """
        request = REQUEST or self.REQUEST
        if self.acl_users.cookie_mode:
            dest = request['URL1']
            request.set('destination', dest)
            return self.acl_users.redirectToLogin(request)
        else:
            user = SecurityManagement.getSecurityManager().getUser()
            if user.getUserName().startswith('Anonymous'):
                request.SESSION.clear()
                request.SESSION.invalidate()
                raise Unauthorized

    login__replaceable__ = REPLACEABLE

    security.declareProtected(View, 'logout')
    def logout(self, REQUEST=None):
        """ logout """
        request = REQUEST or self.REQUEST
        request.SESSION.clear()
        request.SESSION.invalidate()
        if self.acl_users.cookie_mode:
            dest = request['URL1']
            request.set('destination', dest)
            return self.acl_users.logout(request)
        else:
            message = message[NO_LOGOUT]
            action = request['URL1']
            return self.show_message(self, REQUEST=REQUEST, title=message.title,
                                 message=message.text,
                                 action=action
                                )
    logout__replaceable__ = REPLACEABLE

    security.declarePrivate('get_user_data_manager')
    def get_user_data_manager(self):
        v = getattr(self, '_v__user_data_manager', None)
        if not v:
            v = self._v__user_data_manager = createUserDataWrapper(self.acl_users)
        return v

    security.declareProtected(View, 'power_users_list')
    def power_users_list(self, include_names=1, ids=None, roles=[ExpertRole, ModeratorRole]):
        udm = self.get_user_data_manager()
        if not ListType is type(roles):
            roles = [roles]
        if ids is None:
            ids = []
            for role in roles:
                ids.extend(self.users_with_local_role(role))
            ids.extend(udm.user_ids_with_roles(roles))
            remove_duplicates(ids)
        if include_names:
            udm = self.get_user_data_manager()
            names = map(udm.get_user_realname, ids)
            ids = map(None, names, ids)
        ids.sort()
        return ids

    security.declareProtected(View, 'experts_list')
    def experts_list(self, include_names=1, ids=None):
        # """ experts_list(self, include_names=1, ids=None):
        #     return list of ids of experts assigned to this forum
        #     if include_names == 1: return list of tuples (realname, user_id)
        #     if ids is not None: return tuples(realname, user_id) for ids
        return self.power_users_list(include_names, ids, ExpertRole)

    security.declareProtected(View, 'moderators_list')
    def moderators_list(self, include_names=1, ids=None):
        # """ moderators_list(self, include_names=1, ids=None):
        #     return list of ids of moderators assigned to this forum
        #     if include_names == 1: return list of tuples (realname, user_id)
        #     if ids is not None: return tuples(realname, user_id) for ids
        return self.power_users_list(include_names, ids, ModeratorRole)

    security.declareProtected(View, 'expert_roles')
    def expert_roles(self, expert):
        ''' helper method: returns human readable names for expert roles. '''
        udm = self.get_user_data_manager()
        user = udm.get_user(expert)
        valid_roles = ExpertRole, ModeratorRole
        if user:
            roles = filter(lambda x, l=valid_roles:
                           x in l, user.getRolesInContext(self)
                           )
            roles = remove_duplicates(roles)
        else:
            roles = []
        return ", ".join(map(self.get_translation, roles))

    security.declareProtected(View, 'get_user_realname')
    def get_user_realname(self, userid):
        return self.get_user_data_manager().get_user_realname(userid) or "-"

    security.declareProtected(ViewUserInfo, 'edit_user_data_form')
    def edit_user_data_form(self, username=None, REQUEST=None, message=None):
        """ see manage_edit_user_date_form. call edit_user_data_form from user ui """
        return self.manage_edit_user_data_form(username, REQUEST=REQUEST, manage_mode=0, message=message)

    security.declareProtected(ViewUserInfo, 'manage_edit_user_data_form')
    def manage_edit_user_data_form(self, username=None, REQUEST=None, manage_mode=1, message=None):
        """ manage_edit_user_data_form(self, REQUEST):
            check if current user is exists in acl_users and
            return a form to edit userdata """
        auth_user = SecurityManagement.getSecurityManager().getUser()
        auth_username = auth_user.getUserName()
        if not username:
            username = auth_username
        username == auth_username
        if (auth_username == username) or (auth_user.has_permission(Permissions.manage_users, self.acl_users)):
            gdm = self.get_user_data_manager()
            user = gdm.get_user(username)
            if user:
                body = self._manage_edit_user_data_form_body(self,
                                                             REQUEST,
                                                             user=user,
                                                             manage_mode=manage_mode,
                                                             message=message)
                if user.has_permission('View management screens', self) and manage_mode:
                    return self._manage_edit_user_data_form(self,
                                                            REQUEST,
                                                            manage_edit_user_data_form_body=body,
                                                            manage_tabs_message=message)
                else:
                    return self._ui_edit_user_data_form(REQUEST,
                                                        manage_edit_user_data_form_body=body)

        raise Unauthorized


    security.declarePublic('manage_edit_user_data')
    def manage_edit_user_data(self, username, REQUEST=None, manage_mode=0):
        """ manage_edit_user_data(self, username, REQUEST=None):
            update user data for user with id 'username' """
        req = REQUEST or self.REQUEST
        if req.get('password') != req.get('password_confirm'):
            message = "Passwort und Besttigung stimmen nicht berein."
            if manage_mode:
                return self.manage_edit_user_data_form(username, REQUEST, message=message)
            else:
                return self.edit_user_data_form(username, REQUEST, message=message)
        else:
            try:
                gdm = self.get_user_data_manager()
                gdm.update_user_data(username, req) # update_user_data checks for permissions
                message = "Nutzerdaten wurde aktualisiert."
            except Exception, error:
                message = "Failed to update userdata!"
#                import traceback
#                traceback.print_exc()
        if REQUEST:
            auth_user = SecurityManagement.getSecurityManager().getUser()
            if auth_user.allowed(self.manage_edit_users_form) and manage_mode:
                return self.manage_edit_users_form(self, req, manage_tabs_message=message)
            else:
                return self.index_html(REQUEST)


    security.declareProtected(ManageReQueztForum, 'manage_import_user_data_form')
    manage_import_user_data_form = getTemplateFile('templates/ReQueztForum_import_user_data_form', globals())

    security.declareProtected(ManageReQueztForum, 'manage_import_user_data')
    def manage_import_user_data(self, csv_file,
                                has_field_names=0,
                                excel_format=0,
                                REQUEST=None):
        """ manage_import_user_data(self, csv_file, has_field_names=0,
                                    excel_format=0, REQUEST=None):
            import users w/ properties from csv file """
        udm = self.get_user_data_manager()
        try:
            udm.import_user_data(csv_file, has_field_names, excel_format)
        except:
##             import traceback
##             print traceback.print_exc()
            message = "Import failed. Please verify the csv file"
        else:
            message = "Data successfully imported."
        if REQUEST:
            return self.manage_edit_users_form(self, REQUEST, manage_tabs_message=message)

    security.declareProtected(ManageReQueztForum, 'manage_edit_experts')
    def manage_edit_experts(self, expertIds=[], moderatorIds=[], REQUEST=None, RESPONSE=None):
        """ manage_edit_experts(self, usernames, REQUEST=None):
            set local role ExpertRole for usernames. Del users w/o
            any role from localRoles
            return None or HTML """
        dict = {}
        local_superusers = []
        superuser_roles = [ExpertRole, ModeratorRole]
        # get user ids for users with relevant local roles
        for role in superuser_roles:
            local_superusers.extend(self.users_with_local_role(role))
        local_superusers = local_superusers + expertIds + moderatorIds
        remove_duplicates(local_superusers)

        # set new local roles and delete obsolete local roles
        for username in local_superusers:
            roles = {}
            map(roles.setdefault, self.get_local_roles_for_userid(username))
            if username in expertIds:
                roles[ExpertRole] = 1
            elif roles.has_key(ExpertRole):
                del roles[ExpertRole]
            if username in moderatorIds:
                roles[ModeratorRole] = 1
            elif roles.has_key(ModeratorRole):
                del roles[ModeratorRole]
            dict[username] = roles.keys()

        delusers = []
        for username, roles in dict.items():
            if roles:
                self.manage_setLocalRoles(username, roles)
            else:
                delusers.append(username)
        if delusers:
            self.manage_delLocalRoles(delusers)

        if REQUEST:
            RESPONSE = RESPONSE or REQUEST.RESPONSE
            args = {'manage_tabs_message':"Die Daten wurden aktualisiert", 'userListStart':REQUEST.get('userListStart',0)}
            args = map(None, map(str, args.keys()), map(str, args.values()))
            args = "&".join(map("=".join, args))
            url = 'manage_edit_users_form?%s' % args
            RESPONSE.redirect(url)

########
## methods related to moderating and managing the forum
########

    security.declareProtected(ManageReQueztForum, 'manage_edit')
    def manage_edit(self, REQUEST=None, **kw):
        """ edit ReQuezt Forum options  """
        mail_host = kw.get('mail_host')
        if mail_host:
            try:    v=getattr(self, mail_host)
            except:
                return MessageDialog(title='Invalid Mail Host',
                                     message='Cannot find the named mail host!',
                                     action=REQUEST['URL']+'/manage_main'
                                    )
        if mail_host is not None: self.mail_host=mail_host
        moderated = kw.get('moderated')
        if moderated == 'both':
            kw['moderated'] = 1
            kw['mod_comment'] = 1
        elif moderated == 'entries':
            kw['moderated'] = 1
            kw['mod_comment'] =0
        else:
            kw['moderated'] = 0
            kw['mod_comment'] = 0
                    
        self.manage_changeProperties(REQUEST, **kw)
        
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

    security.declareProtected(ModerateEntries, 'manage_review')
    def manage_review(self,ids=[],REQUEST=None):
        """ approve selected entries from a ReQuezt Forum site """
        ids=map(int, ids)
        for id in ids:
            self.set_reviewed(self.data[id].__of__(self))
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

    security.declareProtected(ModerateEntries, 'moderation_process')
    def moderation_process(self, approve=None, delete=None, REQUEST=None):
        """ do the processing for things from the moderation tab """
        if approve:
            self.manage_review(REQUEST['approve'],REQUEST)
        if delete:
            self.manage_delete(REQUEST['delete'],REQUEST)
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

    # Used for Bruce Perens' Batch Moderation Code
    # Indicate that entry has been reviewed
    security.declarePrivate('set_reviewed')
    def set_reviewed(self, item):
        item.reviewed=1
        for t in item.getThread():
            obj=self.data[t]
            if obj.revsub > 0: obj.revsub=obj.revsub-1
        # re-catalog the item now that it's reviewed status has changed.
        item.index()

    # Returns true if there are un-reviewed entries in this site
    security.declareProtected(ModerateEntries, 'unmoderated_entries')
    def unmoderated_entries(self):
        # print "reviewed:",len(self.searchResults({'reviewed': 1 }))
        # print "data:",len(self.data)
        return len(self.searchResults({'reviewed': 1 })) < len(self.data)
    

    security.declareProtected(ModerateEntries, 'manage_delete_entries')
    def manage_delete_entries(self,ids=[],REQUEST=None):
        """ delete selected entries  """
        ids=map(int, ids)
        for id in ids:
            self.delItem(id)
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

    security.declareProtected(ModerateEntries, 'exportXML')
    def exportXML(self, REQUEST=None, RESPONSE=None):
        """ exportXML(self, REQUEST=None, RESPONSE=None)
        Export all entries in the forum object as xml file """
        try:
            from xml.dom import minidom
        except ImportError:
            if REQUEST:
                message = messages[NO_XML_SUPPORT]
                action = "%s/manage_main" % self.absolute_url()
                return self.show_message(self, REQUEST=REQUEST, title=message.title,
                                     message=message.text,
                                     action=action
                                    )
            else:
                raise

        if RESPONSE:
            write = RESPONSE.write
            RESPONSE.setHeader("content-type", "text/html")
            RESPONSE.setHeader("content-disposition", "filename=%s.xml" % url_quote(self.getId()))
            close = None
        else:
            from cStringIO import StringIO
            s = StringIO()
            write = s.write
            close = s.close

        d = minidom.Document()
        entryList = d.createElement("ReQueztEntries")
        for entry in self.entryValues():
            node = entry.exportXML(d)
            entryList.appendChild(node)
        d.appendChild(entryList)
        write(d.toprettyxml())
        d.unlink()
        if close:
            close()

    security.declareProtected(ModerateEntries, 'remind_all')
    def remind_all(self, REQUEST=None):
        """ send default reminders to all overdue messages """
        req = REQUEST or self.REQUEST
        late_entries_filter = self.views.late_entries.getFilter()
        search_results = self(REQUEST=None, **late_entries_filter)
        remind_count = len(search_results)

        try:
            for entry in search_results:
                    ob = self.getobject(entry.data_record_id_)
                    ob.remind(req)
        except:
            LOG(self.meta_type,
                WARNING,
                'remind_all failed',
                'Forum Id: %s' % (self.getId()),
                error=sys.exc_info())

        message = messages[REMIND_ALL]
        message_text = message.text % {'remind_count':remind_count}
        if REQUEST is not None:
            action = "%s/manage_main" % self.absolute_url()
            return self.show_message(self, REQUEST=REQUEST, title=message.title,
                                     message = message_text,
                                     action = action)
        else:
            return message_text



    security.declareProtected('Manage', 'addRandomMessages')
    def addRandomMessages(self, count=100, REQUEST=None, RESPONSE=None):
        """ addRandomMessages(self, count=100): add <count> random messages to forum """
        import RandomDataGenerator
        import random
        from Entries import Entry, Reply

        def addItem(parent, Klass, form_id, form_data):
            def createId(parent=parent, now=int(time())):
                from random import random
                id = now - int(random() * 365 * 86400)
                while parent.data.has_key(id):
                    id += 1
                return id
            id = createId()
            thread=[]
            if not parent.root:
                map(thread.append, parent.thread)
                thread.append(int(parent.getId()))
            item=Klass(id, thread, parent.level+1, form_id, form_data).__of__(parent)
            item.created = id
            item.modified = id
            parent.setItem(id, item, 1)
            return item

        rg = getattr(self, '_v_rg', RandomDataGenerator.RandomDataGenerator())
        self._v_rg = rg
        domains = ['de', 'es', 'uk', 'com', 'org', 'net', 'pt', 'edu', 'gov', 'se']
        experts = self.experts_list(include_names=0)
        topics = self.topics_list(include_names=0)
        count = int(count)
        write = RESPONSE.write
        RESPONSE.setHeader('content-type', 'text/html')
        write("""<html><body>""")
        for n in xrange(count):
            try:
                data = {}
                data['title'] = rg.getRandomText(10)
                data['author'] = rg.getRandomText(3)
                data['email'] = "%s@%s.%s" % (rg.getRandomWord(), rg.getRandomWord(), random.choice(domains))
                data['body'] = rg.getRandomText(150)
                data['body_encoding'] = MSG_ENC_PLAIN
                data['expert'] = random.choice(experts)
                data['topic'] = random.choice(topics)
                data['important'] = int(random.random() + 0.1)
                data['notify'] = int(random.random() + 0.9)
                entry = addItem(self, Entry, 'frmEntry', data)
                expert = data['expert']
                write("<p>Entry: [%i] %s</p>" % (n, data['title']))
                for m in range(0, random.randrange(0, 2)):
                    data = {}
                    data['author'] = expert
                    data['title'] = entry.suggest_title()
                    data['body'] = rg.getRandomText(50)
                    data['body_encoding'] = MSG_ENC_PLAIN
                    addItem(entry, Reply, 'frmReply', data)
                    write("<p>&nbsp;&nbsp;&nbsp;&nbsp;Reply: %s</p>" % data['title'])
            except:
	        pass
#                import traceback
#                traceback.print_exc()
        write("</body></html>")

    security.declareProtected(ManageReQueztForum, 'delete_all_entries')
    def delete_all_entries(self, REQUEST):
        """ clear all entries """
        if REQUEST.get('delete_all_entries') =='delete_all_entries':
            self.data = IOBTree()  # id -> Message
            self._ids = intSet() # ids of children
            self._catalog.clear()
            message = 'This action can be undone using the Undo tab'
            return self.manage_entries(self, REQUEST, manage_tabs_message=message)





InitializeClass(ReQueztForum)

def registerClass(ElementClass, constructors=()):
    '''registerReQueztView(ElementClass, constructors=()) --> Register a new Topic; return None'''
    dict = { 'name': ElementClass.meta_type,
             'action': 'manage_addProduct/ReQuezt/' + constructors[0].__name__ }

    for constructor in constructors:
        setattr(ReQuezt, constructor.__name__, constructor)

    ReQuezt.meta_types = ReQuezt.meta_types + (dict,)
    ReQuezt.selected_meta_types = ReQuezt.selected_meta_types + (dict['name'],)
