# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

import Globals
from Products.ReQuezt.Const import *
from Products.ReQuezt.Utility import getFileIds
from Products.ReQuezt.Skins.ReQueztSkin import ReQueztSkin
from Products.ReQuezt.Skins import registerSkin
from Products.ReQuezt.ActionDefs  import manage_addActionDefContainer
from Products.ReQuezt.ColumnDefs import manage_addColumnDefContainer
from Products.ReQuezt.ColumnDefs import LABEL_TYPE_TEXT, LABEL_TYPE_IMG, LABEL_TYPE_TEXT_AND_IMG

class BasicSkin(ReQueztSkin):
    
    _ACTION_REPLY_CONDITION = """not:options/result_item/reply_count"""
    _ACTION_REMIND_CONDITION = """here/late | python:parent.getobject(options['result_item'].data_record_id_).late()"""
    def __init__(self):
        ReQueztSkin.__init__(self)
        setoption = self.setOption
        setoption('help_contexts', {DEFAULT_VIEW_ID:
                         {'index_html': 'Index',
                          'add_entry_form': 'add_entry',
                          'entry_edit_form': 'edit_entry'},
                         'search':
                         {'index_html': 'suche'}
                         })
        setoption('sort_descriptor', ('id', 'cmp', 'desc'))
        setoption('secondary_sort_descriptor', ('important', 'cmp', 'desc'))

        setoption('actions_order', [ACTION_VIEW, ACTION_REPLY, ACTION_FORWARD, ACTION_EDIT, ACTION_REMIND])



        setoption('indexes',
                  [('author', 'FieldIndex'),
                   ('textToSearch', 'TextIndex'),
                   ('topic', 'FieldIndex'),
                   ('expert', 'FieldIndex'),
                   ('reviewed', 'FieldIndex'),
                   ('date', 'FieldIndex'),
                   ('answered', 'FieldIndex'),
                   ('answered_by', 'KeywordIndex'),
                   ('reply_count', 'FieldIndex'),
                   ('comment_count', 'FieldIndex'),
                   ('important', 'FieldIndex'),
                   ('reminder_count', 'FieldIndex'),
                   ('last_reminder', 'FieldIndex')
                   ]
                  )
        setoption('index_titles',
                  ({'author': 'Author',
                    'important': 'Important', 
                    'textToSearch': LABEL_TYPE_TEXT,
                    'topic': 'Topic',
                    'expert': 'Expert',
                    'reviewed': 'Reviewed',
                    'date': 'Date',
                    'answered': 'Answered',
                    'answered_by': 'Answered by',
                    'reply_count': 'Replies',
                    'comment_count': 'Comments',
                    'reminder_count': 'Reminders',
                    'meta_type': 'Meta Type'}
                   )
                  )

        setoption('meta_data_columns',
                  ['thread_path',
                   'get_expert_name',
                   'get_author_name',
                   'get_author_email',
                   'date_created_string',
                   'date_sort',
                   'reply_count',
                   'comment_count',
                   'has_items',
                   'title',
                   'important',
                   'reminder_count']
                  )

        self.local_templates = [{'id': 'index_html.pt', 'title': 'Content',
                                 'roles': (AnonymousRole, AuthenticatedRole)},
                                {'id': 'add_entry_form', 'title': 'Add Entry',
                                 'roles': (AnonymousRole, ExpertRole, AuthenticatedRole)},
                                {'id': 'add_reply_form', 'title': 'Add Reply',
                                 'roles': (ExpertRole, ModeratorRole, ManagerRole)},
                                {'id': 'add_comment_form', 'title': 'Add Comment',
                                 'roles': (ModeratorRole, ManagerRole)},
                                {'id': 'entry_html', 'title': 'Entry',
                                 'roles': (AnonymousRole, AuthenticatedRole)},
                                {'id': 'reply_html', 'title': 'Reply',
                                 'roles': (AnonymousRole, AuthenticatedRole)},
                                {'id': 'comment_html', 'title': 'Comment',
                                 'roles': (AnonymousRole, AuthenticatedRole)},
                                {'id': 'entry_body', 'title': 'Entry Body',
                                 'roles': (AnonymousRole, AuthenticatedRole)},
                                {'id': 'entry_edit_form', 'title': 'Edit Entry',
                                 'roles': (ExpertRole, ModeratorRole, ManagerRole)},
                                {'id': 'entry_forward_form', 'title': 'Forward Entry',
                                 'roles': (ExpertRole, ModeratorRole, ManagerRole)},
                                {'id': 'entry_remind_form', 'title': 'Send Reminder',
                                 'roles': (ModeratorRole, ManagerRole)},
                                {'id': 'entry_status', 'title': 'Entry Status',
                                 'roles': (ModeratorRole, ManagerRole)}                          
                                ]

        self.topics = [{'id': 'about', 'title': 'About this ReQuezt Forum', 'title_short': 'About...'}]

        self.views = [{'id': DEFAULT_VIEW_ID, 'title': 'All Entries',
                       'title_short': 'All Entries', 'description': 'default_description'},
                      {'id': 'search', 'title': 'Searchresults',
                       'title_short': 'Search', 'filter_method': 'search_getFilter',
                       'description': 'search_description', 'hidden': 1},
                      {'id': 'pending', 'title': 'Pending Entries',
                       'title_short': 'Pending', 'filter_method': 'pending_getFilter',
                       'description': 'pending_description',
                       'roles': (ManagerRole, ExpertRole, ModeratorRole)},
                      {'id': 'overdue_entries', 'title':'Overdue Entries',
                       'title_short': 'Overdue', 'filter_method': 'overdue_entries_getFilter',
                       'description': 'overdue_entries_description',
                       'roles': (ManagerRole, ModeratorRole)}
                      ]

        self.menus = [{'id': 'menu_main', 'title': 'Main Menu',
                       'roles': (AnonymousRole, AuthenticatedRole)},
                      {'id': 'menu_expert', 'title': 'Expert Contextmenu',
                       'roles': (ExpertRole, ModeratorRole, ManagerRole)}
                      ]


        self.property_values = {'title': 'title',
                                'title_short': 'title_short',
                                'forum_email_address': '_email_address'}

        self.authSourceType = 'zodbAuthSource'
        self.propSourceType = 'zodbPropSource'
        self.memberSourceType = 'nullMemberSource'

    def addActionDefs(self, parent, id):
        container = manage_addActionDefContainer(parent, id, id)
        addAction = container.manage_addActionDef
        addAction(ACTION_PREVIOUS, 'Previous', 'string:../${here/previous_entry_id}', 'res/previous.png',
                  'here/previous_entry_id | nothing', 
                  (AuthenticatedRole, AnonymousRole, ExpertRole, ModeratorRole))
        addAction(ACTION_NEXT, 'Next', 'string:../${here/next_entry_id}', 'res/next.png',
                  'here/next_entry_id | nothing', 
                  (AuthenticatedRole, AnonymousRole, ExpertRole, ModeratorRole))
        addAction(ACTION_REPLY, 'Reply', 'string:add_reply_form', 'res/add_reply.png', 
                  self._ACTION_REPLY_CONDITION, (ExpertRole,))
        addAction(ACTION_COMMENT, 'Comment', 'string:add_comment_form', 'res/add_comment.png', 
                  None, (ModeratorRole,))
        addAction(ACTION_FORWARD, 'Forward', 'string:entry_forward_form', 'res/entry_forward.png', None, 
                  (ExpertRole, ModeratorRole))
        addAction(ACTION_EDIT, 'Edit', 'string:entry_edit_form', 'res/entry_edit.png', None, 
                  (ExpertRole, ModeratorRole))
        addAction(ACTION_REMIND, 'Remind',  'string:entry_remind_form', 'res/entry_remind.png',
                  self._ACTION_REMIND_CONDITION,(ModeratorRole,))
        return container

    def addColumnDefs(self, parent, id, catalog_id):
        container = manage_addColumnDefContainer(parent, id, title=id, catalog_id=catalog_id)
        addColumn = container.manage_addColumnDef
        addColumn('important', LABEL_TYPE_IMG, 'Important', '1%',
                  """<img tal:replace="structure
                  python:test(options['result_item'].important, path('here/res/important.png'), chr(160))" />""",
                  image_url='res/important.png')
        addColumn('', LABEL_TYPE_TEXT, 'Title', '51%',
                  """<a tal:define="result_item options/result_item;
base_url options/base_url" tal:attributes="href string:$base_url/${result_item/id}/entry_html"
       tal:content="result_item/title" href="#">Entry->Title</a>""")
        addColumn('', LABEL_TYPE_TEXT_AND_IMG, '', '5%',
                  """<span tal:define ="result_item options/result_item;
                  base_url options/base_url"
                  tal:condition="python:user.has_role('Authenticated', here)"
                  tal:repeat="action python:here.action_defs.getValidActions(result_item=result_item)"><a
                  href="#"
                  tal:attributes="href string:$base_url/${result_item/id}/${action/getMethodId}">
                  <img tal:condition="action/image_url"
                  src="res/action_view.png"
                  border="1"
                  tal:attributes="src action/image_url;
                  alt action/title | nothing" /><span
                  tal:condition="not:action/image_url"
                  tal:replace="string:[${action/title}]">[Aktion]</span></a>&nbsp;
                  </span>""", style="white-space:nowrap")
        addColumn('author', LABEL_TYPE_TEXT, 'Author', '21%',
                  """<span tal:replace="options/result_item/get_author_name | string:Anonym">Author</span>""")
        addColumn('date', LABEL_TYPE_TEXT, 'Date', '21%',
                  """<span tal:replace="python:DateTime(int(options['result_item'].id)).strftime('%d.%m.%Y')">Date</span>""")
        addColumn('reply_count', LABEL_TYPE_TEXT, 'Replies', '1%',
                  """<img src="res/answered.png" tal:attributes="src
                  python:'res/%sanswered.png' % test(options['result_item'].has_items, '','un')" />""", align="center")
        return container

        
registerSkin(BasicSkin.__name__, BasicSkin)
