# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

import os.path
import Globals
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import access_contents_information, manage_properties
from OFS.ObjectManager import REPLACEABLE
from Products.ReQuezt.Const import *
from Products.ReQuezt.Events import *
from Products.ReQuezt.Permissions import *
from Products.ReQuezt.Utility import getFileIds, file_path as product_home

pjoin = os.path.join

class ReQueztSkin:
    _properties = ()

    def __init__(self):
        self._options = {}
        setoption = self.setOption
        
        self.basepath = Globals.package_home(globals())
        datapath = pjoin(self.basepath, "%sFiles" % self.__class__.__name__)
        self.datapath = datapath[len(product_home) + 1:]
        setoption('master_template_id', 'requezt_master.pt')
        
        setoption('help_contexts', {DEFAULT_VIEW_ID: {'some_method': 'a_context_id'}})
        setoption('sort_descriptor', ('id', 'cmp', 'desc'))
        setoption('secondary_sort_descriptor', None)
        setoption('actions', {'<Role>':
                              {'<ACTION_ID>':
                               {'method': '<method_id>',
                                'img': '<img_url>',
                                'alt': '<alt_text>'}
                               }
                              })
        setoption('actions_order', [ACTION_VIEW, ACTION_REPLY, ACTION_FORWARD, ACTION_EDIT, ACTION_REMIND])
        
        setoption('indexes', []) # [('<id>', '<IndexType>')]
        setoption('index_titles', {'<index_id>': '<Index Title>'})
        setoption('index_headers', [{'sort_key':
                                     '<index_id>',
                                     'type': '[img|text]',
                                     'value': '<url>|<text>',
                                     'width': '<int[%]>'}
                                    ]
                  )

        setoption('meta_data_columns', []) # ['<id>']

        self.forms_ids = getFileIds(pjoin(product_home, pjoin(self.getPath('forms'), '*.xml')))
        
        self.notifications_ids = ['default', ENTRY_ADDED, REPLY_ADDED, COMMENT_ADDED, REMIND_ENTRY,
                                  MOD_FORWARD_GONE, MOD_FORWARD_NEW, FORWARD]

        self.local_templates = [{'id': '<id>', 'title': '<title>',
                                 'roles': ()}]
        self.global_templates = getFileIds(pjoin(product_home, pjoin(self.getPath('global_templates'), '*')))

        self.topics = [{'id': '<id>', 'title': '<title>', 'title_short': '<title_short>'}]

        self.views = [{'id': '<id>', 'title': '<title>',
                       'title_short': '<title short>', 'description': '<description_id>',
                       'roles': ()}]

        self.menus = [{'id': '<id>', 'title': '<title>',
                       'roles': ()}]


        self.property_values = {'title': 'title',
                                'title_short': 'title_short',
                                'forum_email_address': '_default_email_address'}
        
        self.authSourceType = 'zodbAuthSource'
        self.propSourceType = 'zodbPropSource'
        self.memberSourceType = 'nullMemberSource'
        
    
    security = ClassSecurityInfo()

    security.declareProtected(access_contents_information, 'getOption')
    def getOption(self, id):
        return self._options.get(id)
    
    security.declareProtected(manage_properties, 'setOption')
    def setOption(self, id, value):
        self._options[id] = value
    
    security.declareProtected(View, 'get_index_headers')
    def get_index_headers(self):
        return self.index_headers
    get_index_headers__replaceable__ = REPLACEABLE
    
    security.declareProtected(View, 'get_valid_actions')
    def get_valid_actions(self, user, result_item):
        roles = user.getRolesInContext(self)
        valid_actions = {}
        actions = self.actions
        for role in roles:
            if actions.has_key(role):
                valid_actions.update(actions[role])
        valid_actions_list = filter(None, map(valid_actions.get, self.actions_order))
        return valid_actions_list

    security.declarePrivate('getPath')
    def getPath(self, objectType):
        return pjoin(self.datapath, objectType)

    def addActionDefs(self, ob, id):
        """ add ActionDefs for skin to ob using id """
        raise "Not Implemented"

    def addColumnDefs(self, ob, id):
        """ add ColumnDefs for skin to ob using id """
        raise "Not Implemented"
    

Globals.InitializeClass(ReQueztSkin)
