# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

from types import *
from AccessControl.unauthorized import Unauthorized
from AccessControl import SecurityManagement
from Permissions import *
from Utility import intersect

class AbstractUserDataWrapper:
    """ AbstractUserDataWrapper: Proxy class to access a user data source """

    def __init__(self, data_source=None):
        self._data_source = data_source

    def get_user(self, id):
        pass
    def set_data_source(self, data_source):
        self._data_source = data_source
    def initialize_data_source(self, parent, data_source_id, host_user_folder=None):
        pass
    def get_user_ids_list(self):
        pass
    def users_with_roles(self, roles):
        pass
    def get_user_name(self, id):
        pass
    def get_user_email(self, id):
        pass
    def get_user_topics(self, id):
        pass
    def set_user_name(self, id, name):
        pass
    def set_user_email(self, id, email):
        pass
    def set_user_topics(self, id, topics):
        pass
    def add_user(self, id, user_name, email, topics=[]):
        pass
    def delete_users(self, ids):
        pass

class exUserFolderUserDataWrapper(AbstractUserDataWrapper):
    default_schema = "username password realname email"
    
    def get_user(self, id):
        ob = self._data_source.getUser(id)
        return ob

    def user_exists(self, id):
        if self.get_user(id):
            return 1
        else:
            return 0
        
    def get_users_ids_list(self):
        return self._data_source.getUserNames()
    
    def users_with_roles(self, roles):
        if not type(roles) in (ListType, TupleType):
            roles = (roles,)
        result = []
        for user in self._data_source.getUsers():
            if intersect(roles, user.getRoles()):
                result.append(user)
        return result
    
    def user_ids_with_roles(self, roles):
        users = self.users_with_roles(roles)
        return map(lambda x: x.getUserName(), users)
            
        
    
    def get_user_realname(self, id):
        ob = self.get_user(id)
        if ob:
            username = ob.getProperty('realname')
            username = username or id
        else:
            username = id
        return username
    
    def get_user_email(self, id):
        ob = self.get_user(id)
        if ob:
            user_email = ob.getProperty('email')
        else:
            user_email = None
        return user_email
    
    def update_user_data(self, id, data={}):
        from types import DictType
        securityManager = SecurityManagement.getSecurityManager()
        u = securityManager.getUser()
        u_id = u.getUserName()
        if (u_id == id) or u.has_role([ManagerRole, ModeratorRole], self):
            if not DictType is type(data):
                # make a real dict from data, so we can del items 
                _data = data
                data = {}
                map(data.setdefault, _data.keys(), _data.values())
            password = None
            roles = None
            domains = None
            has = data.has_key
            get = data.get
            if has('password'):
                password = get('password')
                del data['password']
            if has('roles'):
                roles = get('roles')
                del data['roles']
            if has('domains'):
                domains = get('domains')
                del data['domains']
            self._data_source.userFolderEditUser(id, password, roles, domains=domains, **data)
        else:
            raise Unauthorized


    def import_user_data(self, data, has_field_names=0, excel_format=0):
        """ import_user_data(self, data, has_field_names=0, excel_format=0):
            import user data from a csv file. data may be a string or an open 
            file or stream, which will be read and closed """
        import ASV
        from types import ListType, DictType
        
        default_schema = ['username', 'password', 'roles', 'realname', 'email']
        reserved_keys = ['username', 'password', 'roles']
        schema_length = len(default_schema)

        if hasattr(data, 'read') and callable(data.read):
            f = data
            data = data.read()
            f.close()
        asv = ASV.ASV()
        if excel_format:
            importer = ASV.WinColonSV()
        else:
            importer = ASV.CSV()
            
        asv.input(data, importer, has_field_names = has_field_names)
        for row in asv:
            userdata = {}
            if DictType is type(row):
                userdata.update(row)
            else:
                row = row[:schema_length]
                row.reverse()
                for key in default_schema:
                    value = row.pop()
                    if key in reserved_keys:
                        userdata[key] = value
                    else:
                        userdata['user_%s' % key] = value
            username = userdata['username']

            u = self.get_user(username)
            password = userdata.get('password', None)
            roles = userdata.get('roles', None)
            if not ListType is type(roles): roles = [roles]
            roles = filter(None, roles)
            domains = userdata.get('domains', None)
            if not ListType is type(domains): domains = [domains]
            domains = filter(None, domains)
            try:
                del userdata['username']
                del userdata['password']
                del userdata['roles']
                del userdata['domains']
            except: pass
            try:
                if u:
                    self._data_source.userFolderEditUser(username, password, roles, domains, **userdata)
                else:
                    self._data_source.userFolderAddUser(username, password, roles, domains, **userdata)
            except:
	        pass
#                import traceback
#                traceback.print_exc()
                
def createUserDataWrapper(data_source=None):
    ob = exUserFolderUserDataWrapper(data_source)
    return ob
